#include <iostream.h>
#include "sys_signal.hh"

/****************************************************************************
**                              Signal_Set                                 **
/***************************************************************************/



/****************************************************************************
**                           Signal_Action                                 **
/***************************************************************************/

#ifdef __osf__
Signal_Action::Signal_Action(void (*handler)(int),int flags)
{
  action.sa_handler = handler;
  sigemptyset(&action.sa_mask);
  action.sa_flags   = 0x00;       // <-----------------------------------------
}
#endif

#ifdef __sun__
Signal_Action::Signal_Action(void (*handler)(int),int flags)
{
  action.sa_handler = handler;
  sigemptyset(&action.sa_mask);
  action.sa_flags   = 0x00;       // <-----------------------------------------
}
#endif

#ifdef __linux__
Signal_Action::Signal_Action(void (*handler)(int),int flags)
{
  action.sa_handler = handler;
  sigemptyset(&action.sa_mask);
  if((flags & SYS_SA_SINGLE) == SYS_SA_SINGLE)
    action.sa_flags   = SA_ONESHOT;
  else if((flags & SYS_SA_MULTI) == SYS_SA_MULTI)
    action.sa_flags   = SA_RESTART;
  else
    action.sa_flags   = 0x00;
  
  action.sa_flags   = SA_RESTART;
}
#endif


Signal_Action::~Signal_Action()
{
  ;
}

int Signal_Action::Mask(int sig)
{
  return sigaddset(&action.sa_mask,sig);
}


/****************************************************************************
**                           Signal_Manager                                **
/***************************************************************************/

int Signal_Manager::Send(pid_t process,int sig)
{
  return kill(process,sig);
}

int Signal_Manager::SetAction(int sig,Signal_Action& sa)
{
  return sigaction(sig,&(sa.action),NULL);
}


int Signal_Manager::SetDefaultAction(int sig)
{
  struct sigaction sa;

#ifdef __linux__  
  sa.sa_handler = SIG_DFL;
  sigemptyset(&sa.sa_mask);
  sa.sa_flags   = SA_RESTART;
#endif 
  return sigaction(sig,&sa,NULL);
}


int Signal_Manager::Block(int sig)
{
  sigset_t set;
  sigemptyset(&set);
  sigaddset(&set,sig);
  int err = sigprocmask(SIG_BLOCK,&set,NULL);
  if(err < 0)
    {
      cerr << "Signal_Manager: blocking error\n";
      return err;
    }
  else
    return 0;
}

int Signal_Manager::UnBlock(int sig)
{
  sigset_t set;
  sigemptyset(&set);
  sigaddset(&set,sig);
  int err = sigprocmask(SIG_UNBLOCK,&set,NULL);
  if(err < 0)
    {
      cerr << "Signal_Manager: unblocking error\n";
      return err;
    }
  else
    return 0;
}
